// ignore_for_file: avoid_print, unnecessary_string_interpolations, unnecessary_brace_in_string_interps

import 'package:flutter/services.dart';

class TLNumberTextInputFormatter extends TextInputFormatter {
  /// [maxIntegerLength]限定整数的最大位数，为null时不限
  /// [maxDecimalLength]限定小数点的最大位数，为null时不限
  /// [isAllowDecimal]是否可以为小数，默认是可以为小数，也就是可以输入小数点
  final int? _maxIntegerLength;
  final int? _maxDecimalLength;
  final bool _isAllowDecimal;

  TLNumberTextInputFormatter({int? maxIntegerLength, int? maxDecimalLength, bool isAllowDecimal = true})
      : _maxIntegerLength = maxIntegerLength,
        _maxDecimalLength = maxDecimalLength,
        _isAllowDecimal = isAllowDecimal;

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    String? value = newValue.text.trim();

    ///去掉前后空格
    int selectionIndex = newValue.selection.end;
    if (_isAllowDecimal) {
      if (value == '.') {
        value = '0.';
        selectionIndex++;
      } else if (value != '' && _isToDoubleError(value)) {
        ///不是double输入数据
        return _oldTextEditingValue(oldValue);
      }

      ///包含小数点
      if (value.contains('.')) {
        int pointIndex = value.indexOf('.');
        String beforePoint = value.substring(0, pointIndex);
        print('$beforePoint');
        String afterPoint = value.substring(pointIndex + 1, value.length);
        print('$afterPoint');

        ///小数点前面没内容补0
        if (beforePoint.isEmpty) {
          value = '0.${afterPoint}';
          selectionIndex++;
        } else {
          ///限定整数位数
          if (null != _maxIntegerLength) {
            if (beforePoint.length > _maxIntegerLength!) {
              return _oldTextEditingValue(oldValue);
            }
          }
        }

        ///限定小数点位数
        if (null != _maxDecimalLength) {
          if (afterPoint.length > _maxDecimalLength!) {
            return _oldTextEditingValue(oldValue);
          }
        }
      } else {
        ///限定整数位数
        if (null != _maxIntegerLength) {
          if (value.length > _maxIntegerLength!) {
            return _oldTextEditingValue(oldValue);
          }
        }
      }
    } else {
      if (value.contains('.') || (value != '' && _isToDoubleError(value)) || (null != _maxIntegerLength && value.length > _maxIntegerLength!)) {
        return _oldTextEditingValue(oldValue);
      }
    }

    return TextEditingValue(
      text: value,
      selection: TextSelection.collapsed(offset: selectionIndex),
    );
  }

  ///返回旧的输入内容
  TextEditingValue _oldTextEditingValue(TextEditingValue oldValue) {
    return TextEditingValue(
      text: oldValue.text,
      selection: TextSelection.collapsed(offset: oldValue.selection.end),
    );
  }

  ///输入内容不能解析成double
  bool _isToDoubleError(String value) {
    try {
      double.parse(value);
    } catch (e) {
      return true;
    }
    return false;
  }
}

///忽略表情
class TLBlacklistingTextInputFormatter extends TextInputFormatter {
  final _regExp = r"^[^\\u0020-\\u007E\\u00A0-\\u00BE\\u2E80-\\uA4CF\\uF900-\\uFAFF\\uFE30-\\uFE4F\\uFF00-\\uFFEF\\u0080-\\u009F\\u2000-\\u201f\r\n]+$";

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

///只能输入数字和小写字母
class TLOnlyInputNumberAndLowWorkFormatter extends TextInputFormatter {
  final _regExp = r"^[a-z0-9_]+$";
  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

///只能输入数字和字母
class TLOnlyInputNumberAndWorkFormatter extends TextInputFormatter {
  final _regExp = r"^[A-Za-z0-9_]+$";
  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

///只能输入字母
class TLOnlyInputWorkFormatter extends TextInputFormatter {
  final _regExp = r"^[A-Za-z]+$";
  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

///忽略特殊字符
class TLIgnoreOtherFormatter extends TextInputFormatter {
  final _regExp = r"^[\u4E00-\u9FA5A-Za-z0-9_]+$";

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

class NumericalRangeFormatter extends TextInputFormatter {
  final double min;
  final double max;
  final int decimalCount;

  NumericalRangeFormatter({required this.min, required this.max, required this.decimalCount});

  @override
  TextEditingValue formatEditUpdate(
    TextEditingValue oldValue,
    TextEditingValue newValue,
  ) {
    num? value = num.tryParse(newValue.text);
    if(value == null) {
      return newValue;
    } else {
      
      if (value < min) {
        String tt = min.toStringAsFixed(decimalCount);
        return TextEditingValue(
          text: tt,
          selection: TextSelection.collapsed(offset: tt.length),
        );
      } else {
        if(value > max) {
          return oldValue;
        } else {
          String sValue = newValue.text.trim();
          int pointIndex = sValue.indexOf('.');
          if (pointIndex > -1) {
            String afterPoint = sValue.substring(pointIndex + 1, sValue.length);
            if (afterPoint.length > decimalCount) {
              return oldValue;
            }
          }
          return newValue;
        }
      }
    }
  }
}